Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaCompile.groovy
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaCompile.groovy	Wed Oct 29 22:22:22 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaCompile.groovy	Wed Oct 29 22:22:22 EST 2008
@@ -0,0 +1,74 @@
+package org.gradle.api.plugins.scala
+
+import org.slf4j.Logger
+import org.slf4j.LoggerFactory
+
+class AntScalaCompile {
+    private static Logger logger = LoggerFactory.getLogger(AntScalaCompile)
+
+    private final AntBuilder ant
+    private final List<File> bootclasspathFiles
+    private final List<File> extensionDirs
+
+    def AntScalaCompile(AntBuilder ant) {
+        this.ant = ant
+        this.bootclasspathFiles = []
+        this.extensionDirs = []
+    }
+
+    def AntScalaCompile(AntBuilder ant, List<File> bootclasspathFiles, List<File> extensionDirs) {
+        this.ant = ant
+        this.bootclasspathFiles = bootclasspathFiles
+        this.extensionDirs = extensionDirs
+    }
+
+    void execute(List<File> sourceDirs, List<String> includes, List<String> excludes, File targetDir,
+                 List<File> classpathFiles, ScalaCompileOptions compileOptions) {
+
+        ant.mkdir(dir: targetDir.absolutePath)
+
+        Map options = ['destDir': targetDir] + compileOptions.optionMap()
+        String taskName = compileOptions.useCompileDaemon ? 'fsc' : 'scalac'
+        if (logger.isInfoEnabled()) {
+            StringBuilder builder = new StringBuilder()
+            builder.append("Compiling scala with ${taskName} from source ${sourceDirs}\n")
+            builder.append("    options = ${options}\n")
+            builder.append("    classpath = ${classpathFiles}\n")
+            if (includes) {
+                builder.append("    includes = ${includes}\n")
+            }
+            if (excludes) {
+                builder.append("    excludes = ${excludes}\n")
+            }
+            if (bootclasspathFiles) {
+                builder.append("    bootclasspath = ${bootclasspathFiles}\n")
+            }
+            if (extensionDirs) {
+                builder.append("    extDirs = ${extensionDirs}\n")
+            }
+            logger.info(builder.toString())
+        }
+
+        ant."${taskName}"(options) {
+            sourceDirs.each {dir ->
+                src(location: dir)
+            }
+            bootclasspathFiles.each {file ->
+                bootclasspath(location: file)
+            }
+            extensionDirs.each {dir ->
+                extdirs(location: dir)
+            }
+            classpathFiles.each {file ->
+                classpath(location: file)
+            }
+            includes.each {pattern ->
+                include(name: pattern)
+            }
+            excludes.each {pattern ->
+                exclude(name: pattern)
+            }
+        }
+    }
+
+}
Index: trunk/src/test/groovy/org/gradle/api/plugins/scala/ScalaPluginTest.groovy
===================================================================
--- trunk/src/test/groovy/org/gradle/api/plugins/scala/ScalaPluginTest.groovy	Wed Oct 29 22:24:52 EST 2008
+++ trunk/src/test/groovy/org/gradle/api/plugins/scala/ScalaPluginTest.groovy	Wed Oct 29 22:24:52 EST 2008
@@ -0,0 +1,45 @@
+package org.gradle.api.plugins.scala
+
+import org.gradle.api.Project
+import org.gradle.api.internal.project.PluginRegistry
+import org.gradle.api.tasks.javadoc.Javadoc
+import org.gradle.util.HelperUtil
+import static org.gradle.util.WrapUtil.toSet
+import static org.hamcrest.Matchers.*
+import static org.junit.Assert.*
+import org.junit.Test
+import org.gradle.api.plugins.JavaPlugin
+
+public class ScalaPluginTest {
+
+    @Test
+    public void testApply() {
+        Project project = HelperUtil.createRootProject(new File('path', 'root'))
+        ScalaPlugin scalaPlugin = new ScalaPlugin()
+        scalaPlugin.apply(project, new PluginRegistry())
+
+        def configuration = project.dependencies.configurations[JavaPlugin.COMPILE]
+        assertThat(configuration.extendsFrom, equalTo(toSet(ScalaPlugin.SCALA)))
+        assertFalse(configuration.visible)
+        assertFalse(configuration.transitive)
+
+        configuration = project.dependencies.configurations[ScalaPlugin.SCALA]
+        assertThat(configuration.extendsFrom, equalTo(toSet()))
+        assertFalse(configuration.visible)
+        assertFalse(configuration.transitive)
+
+        def task = project.tasks[JavaPlugin.COMPILE]
+        assertThat(task, instanceOf(ScalaCompile.class))
+        assertThat(task.srcDirs, hasItems(project.convention.plugins.java.srcDirs as Object[]))
+        assertThat(task.scalaSourceDirs, hasItems(project.convention.plugins.scala.scalaSrcDirs as Object[]))
+
+        task = project.tasks[JavaPlugin.TEST_COMPILE]
+        assertThat(task, instanceOf(ScalaCompile.class))
+        assertThat(task.srcDirs, hasItems(project.convention.plugins.java.testSrcDirs as Object[]))
+        assertThat(task.scalaSourceDirs, hasItems(project.convention.plugins.scala.scalaTestSrcDirs as Object[]))
+
+        task = project.tasks[ScalaPlugin.SCALADOC]
+        assertThat(task, instanceOf(ScalaDoc.class))
+        assertThat(task.destinationDir, equalTo(project.convention.plugins.scala.scalaDocDir))
+    }
+}
\ No newline at end of file
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaPlugin.java
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaPlugin.java	Wed Oct 29 22:18:49 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaPlugin.java	Wed Oct 29 22:18:49 EST 2008
@@ -0,0 +1,110 @@
+package org.gradle.api.plugins.scala;
+
+import org.gradle.api.Project;
+import org.gradle.api.Task;
+import org.gradle.api.internal.project.PluginRegistry;
+import org.gradle.api.plugins.*;
+import org.gradle.api.tasks.ConventionValue;
+import org.gradle.api.tasks.compile.Compile;
+import org.gradle.api.tasks.testing.ForkMode;
+import org.gradle.api.tasks.testing.Test;
+import org.gradle.util.GUtil;
+
+import java.util.Arrays;
+import java.util.Map;
+
+public class ScalaPlugin extends JavaPlugin {
+    public static final String SCALADOC = "scaladoc";
+    public static final String SCALA = "scala";
+    private static final String DEFINE_TASK = "defineScalaAnt";
+
+    @Override
+    public void apply(Project project, PluginRegistry pluginRegistry, Map customValues) {
+        pluginRegistry.apply(JavaPlugin.class, project, pluginRegistry, customValues);
+        ScalaPluginConvention scalaPluginConvention = new ScalaPluginConvention(project, customValues);
+        project.getConvention().getPlugins().put("scala", scalaPluginConvention);
+
+        configureDefine(project);
+        configureCompile(project);
+        configureTestCompile(project);
+
+        ((Test) project.task(TEST)).getOptions().getForkOptions().setForkMode(ForkMode.ONCE);
+
+        configureScaladoc(project);
+
+        project.getDependencies().addConfiguration(SCALA).setVisible(false).setTransitive(false);
+        project.getDependencies().configuration(JavaPlugin.COMPILE).extendsFrom(SCALA);
+    }
+
+    private void configureDefine(Project project) {
+        ScalaDefine defineTask = (ScalaDefine) project.createTask(
+                GUtil.map("type", ScalaDefine.class, "dependsOn", RESOURCES, "overwrite", true), DEFINE_TASK);
+        defineTask.conventionMapping(GUtil.map(
+                "scalaLibraryDir", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return scala(convention).getScalaLibraryDir();
+                    }
+                }));
+    }
+
+    private void configureScaladoc(Project project) {
+        ScalaDoc docTask = (ScalaDoc) project.createTask(
+                GUtil.map("type", ScalaDoc.class, "dependsOn", DEFINE_TASK), SCALADOC);
+        docTask.conventionMapping(GUtil.map(
+                "destinationDir", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return scala(convention).getScalaDocDir();
+                    }
+                },
+                "scalaSourceDirs", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return scala(convention).getScalaSrcDirs();
+                    }
+                },
+                "scalaClasspath", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return task.getProject().getDependencies().resolve(SCALA);
+                    }
+                }));
+    }
+
+    private void configureCompile(Project project) {
+        ScalaCompile compileTask = (ScalaCompile) project.createTask(
+                GUtil.map("type", ScalaCompile.class, "dependsOn", Arrays.asList(DEFINE_TASK, RESOURCES), "overwrite", true), COMPILE);
+        configureCompile(compileTask, DefaultConventionsToPropertiesMapping.COMPILE);
+        compileTask.conventionMapping(GUtil.map(
+                "scalaSourceDirs", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return scala(convention).getScalaSrcDirs();
+                    }
+                },
+                "scalaClasspath", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return task.getProject().getDependencies().resolve(SCALA);
+                    }
+                }));
+    }
+
+    private void configureTestCompile(Project project) {
+        ScalaCompile testCompileTask = (ScalaCompile) project.createTask(
+                GUtil.map("type", ScalaCompile.class, "dependsOn", Arrays.asList(DEFINE_TASK, TEST_RESOURCES), "overwrite", true), TEST_COMPILE);
+        configureTestCompile(testCompileTask, (Compile) project.task(COMPILE),
+                DefaultConventionsToPropertiesMapping.TEST_COMPILE);
+        testCompileTask.conventionMapping(GUtil.map(
+                "scalaSourceDirs", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return scala(convention).getScalaTestSrcDirs();
+                    }
+                },
+                "scalaClasspath", new ConventionValue() {
+                    public Object getValue(Convention convention, Task task) {
+                        return task.getProject().getDependencies().resolve(SCALA);
+                    }
+                }));
+    }
+
+    private ScalaPluginConvention scala(Convention convention) {
+        return (ScalaPluginConvention) convention.getPlugins().get("scala");
+    }
+
+}
Index: trunk/src/toplevel/plugin.properties
===================================================================
--- trunk/src/toplevel/plugin.properties	(revision 1035)
+++ trunk/src/toplevel/plugin.properties	Tue Oct 21 22:29:13 EST 2008
@@ -1,3 +1,4 @@
 java=org.gradle.api.plugins.JavaPlugin
 groovy=org.gradle.api.plugins.GroovyPlugin
-war=org.gradle.api.plugins.WarPlugin
\ No newline at end of file
+war=org.gradle.api.plugins.WarPlugin
+scala=org.gradle.api.plugins.scala.ScalaPlugin
Index: trunk/src/samples/scalaproject/api/src/main/scala/org/gradle/sample/impl/PersonImpl.scala
===================================================================
--- trunk/src/samples/scalaproject/api/src/main/scala/org/gradle/sample/impl/PersonImpl.scala	Sun Oct 26 20:11:05 EST 2008
+++ trunk/src/samples/scalaproject/api/src/main/scala/org/gradle/sample/impl/PersonImpl.scala	Sun Oct 26 20:11:05 EST 2008
@@ -0,0 +1,11 @@
+package org.gradle.sample.impl
+
+import org.gradle.sample.api.Person
+
+/**
+ * Immutable implementation of {@link Person}.
+ */
+class PersonImpl(val names: List[String]) extends Person
+{
+
+}
Index: trunk/src/test/groovy/org/gradle/api/plugins/scala/ScalaPluginConventionTest.groovy
===================================================================
--- trunk/src/test/groovy/org/gradle/api/plugins/scala/ScalaPluginConventionTest.groovy	Wed Oct 29 22:16:08 EST 2008
+++ trunk/src/test/groovy/org/gradle/api/plugins/scala/ScalaPluginConventionTest.groovy	Wed Oct 29 22:16:08 EST 2008
@@ -0,0 +1,64 @@
+package org.gradle.api.plugins.scala
+
+import org.gradle.api.plugins.AbstractPluginConventionTest
+import org.gradle.api.plugins.JavaPluginConvention
+import static org.hamcrest.Matchers.*
+import static org.junit.Assert.*
+import org.junit.Before
+import org.junit.Test
+
+public class ScalaPluginConventionTest extends AbstractPluginConventionTest {
+    private ScalaPluginConvention scalaConvention
+    private JavaPluginConvention javaConvention
+
+    Class getType() {
+        ScalaPluginConvention
+    }
+
+    Map getCustomValues() {
+        [scalaSrcDirNames: ['newSourceRootName']]
+    }
+
+    @Before public void setUp() {
+        super.setUp()
+        javaConvention = new JavaPluginConvention(project, [:])
+        project.convention.plugins.java = javaConvention
+        scalaConvention = new ScalaPluginConvention(project, [:])
+    }
+
+    @Test public void testScalaConvention() {
+        assertEquals(['main/scala'], scalaConvention.scalaSrcDirNames)
+        assertEquals(['test/scala'], scalaConvention.scalaTestSrcDirNames)
+        assertEquals([], scalaConvention.floatingScalaSrcDirs)
+        assertEquals([], scalaConvention.floatingScalaTestSrcDirs)
+    }
+
+    @Test
+    public void testScalaDefaultDirs() {
+        checkScalaDirs(project.srcRootName)
+    }
+
+    @Test
+    public void testScalaDynamicDirs() {
+        project.srcRootName = 'mysrc'
+        project.buildDirName = 'mybuild'
+        checkScalaDirs(project.srcRootName)
+    }
+
+    @Test
+    public void testScalaDocDirUsesJavaConventionToDetermineDocsDir() {
+        assertThat(scalaConvention.scalaDocDir, equalTo(new File(javaConvention.docsDir, "scaladoc")))
+
+        scalaConvention.scalaDocDirName = "other-dir"
+        assertThat(scalaConvention.scalaDocDir, equalTo(new File(javaConvention.docsDir, "other-dir")))
+    }
+
+    private void checkScalaDirs(String srcRootName) {
+        scalaConvention.floatingScalaSrcDirs << 'someScalaSrcDir' as File
+        scalaConvention.floatingScalaTestSrcDirs << 'someScalaTestSrcDir' as File
+        assertEquals([new File(project.srcRoot, scalaConvention.scalaSrcDirNames[0])] + scalaConvention.floatingScalaSrcDirs,
+                scalaConvention.scalaSrcDirs)
+        assertEquals([new File(project.srcRoot, scalaConvention.scalaTestSrcDirNames[0])] + scalaConvention.floatingScalaTestSrcDirs,
+                scalaConvention.scalaTestSrcDirs)
+    }
+}
Index: trunk/src/samples/scalaproject/settings.gradle
===================================================================
--- trunk/src/samples/scalaproject/settings.gradle	Tue Oct 21 21:09:43 EST 2008
+++ trunk/src/samples/scalaproject/settings.gradle	Tue Oct 21 21:09:43 EST 2008
@@ -0,0 +1,1 @@
+include "api"
Index: trunk/src/samples/scalaproject/api/build.gradle
===================================================================
--- trunk/src/samples/scalaproject/api/build.gradle	Wed Oct 29 22:19:06 EST 2008
+++ trunk/src/samples/scalaproject/api/build.gradle	Wed Oct 29 22:19:06 EST 2008
@@ -0,0 +1,10 @@
+usePlugin('scala')
+
+targetCompatibility = 1.5
+
+// Set this to where scala is installed
+// FIXME: Get scala library from dependencies
+File scalaHome = new File('/usr/java/tools/scala')
+scalaLibraryDir = new File(scalaHome, 'lib')
+
+// FIXME: Add scala library to compile dependencies
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaCompile.java
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaCompile.java	Wed Oct 29 21:34:51 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaCompile.java	Wed Oct 29 21:34:51 EST 2008
@@ -0,0 +1,122 @@
+package org.gradle.api.plugins.scala;
+
+import org.gradle.api.*;
+import org.gradle.api.tasks.compile.Compile;
+import org.gradle.util.GUtil;
+
+import java.io.File;
+import java.util.ArrayList;
+import java.util.List;
+
+/**
+ * Task to perform scala compilation.
+ */
+public class ScalaCompile extends Compile {
+
+    /**
+     * Directories containing scala source files.
+     */
+    private List<File> scalaSourceDirs;
+
+    /**
+     * Files or directories to add to the compilation classpath.
+     */
+    private List<File> scalaClasspath;
+
+    /**
+     * Include pattern for which scala files should be compiled (e.g. '**&#2F;org/gradle/package1/')).
+     * This pattern is added as an nested include the scalac task.
+     */
+    private List<String> scalaIncludes;
+
+    /**
+     * Exclude pattern for which files should be compiled (e.g. '**&#2F;org/gradle/package2/A*.java').
+     * This pattern is added as an nested exclude the scalac task.
+     */
+    private List<String> scalaExcludes;
+
+    private AntScalaCompile antScalaCompile;
+
+    private ScalaCompileOptions scalaCompileOptions = new ScalaCompileOptions();
+
+    public ScalaCompile(Project project, String name) {
+        super(project, name);
+        setActions(new ArrayList<TaskAction>());
+        doFirst(new TaskAction() {
+            public void execute(Task task) {
+                compile(task);
+            }
+        });
+    }
+
+    public AntScalaCompile getAntScalaCompile() {
+        if (antScalaCompile == null) {
+            antScalaCompile = new AntScalaCompile(getAnt());
+        }
+        return antScalaCompile;
+    }
+
+    public void setAntScalaCompile(AntScalaCompile antScalaCompile) {
+        this.antScalaCompile = antScalaCompile;
+    }
+
+    public List getScalaSourceDirs() {
+        return (List) conv(scalaSourceDirs, "scalaSourceDirs");
+    }
+
+    public void setScalaSourceDirs(List<File> scalaSourceDirs) {
+        this.scalaSourceDirs = scalaSourceDirs;
+    }
+
+    public List<File> getScalaClasspath() {
+        List<File> files = (List<File>) conv(scalaClasspath, "scalaClasspath");
+
+        // FIXME: remove this - use dependencies
+        List<File> classpath = new ArrayList<File>(files);
+        classpath.add(new File("/usr/java/tools/scala/lib/scala-library.jar"));
+        return classpath;
+    }
+
+    public void setScalaClasspath(List<File> scalaClasspath) {
+        this.scalaClasspath = scalaClasspath;
+    }
+
+    public List<String> getScalaIncludes() {
+        return (List<String>) conv(scalaIncludes, "scalaIncludes");
+    }
+
+    public void setScalaIncludes(List<String> scalaIncludes) {
+        this.scalaIncludes = scalaIncludes;
+    }
+
+    public List<String> getScalaExcludes() {
+        return (List<String>) conv(scalaExcludes, "scalaExcludes");
+    }
+
+    public void setScalaExcludes(List<String> scalaExcludes) {
+        this.scalaExcludes = scalaExcludes;
+    }
+
+    public ScalaCompileOptions getScalaCompileOptions() {
+        return scalaCompileOptions;
+    }
+
+    public void setScalaCompileOptions(ScalaCompileOptions scalaCompileOptions) {
+        this.scalaCompileOptions = scalaCompileOptions;
+    }
+
+    @Override
+    protected void compile(Task task) {
+
+        if (!GUtil.isTrue(getTargetCompatibility())) {
+            throw new InvalidUserDataException("The targetCompatibility must be set!");
+        }
+
+        List<File> existingSourceDirs = existentDirsFilter.checkDestDirAndFindExistingDirsAndThrowStopActionIfNone(
+                getDestinationDir(), getScalaSourceDirs());
+
+        getAntScalaCompile().execute(existingSourceDirs, getScalaIncludes(), getScalaExcludes(), getDestinationDir(),
+                getScalaClasspath(), getScalaCompileOptions());
+    }
+
+}
\ No newline at end of file
Index: trunk/src/samples/scalaproject/api/src/main/scala/org/gradle/sample/api/Person.scala
===================================================================
--- trunk/src/samples/scalaproject/api/src/main/scala/org/gradle/sample/api/Person.scala	Sun Oct 26 19:41:33 EST 2008
+++ trunk/src/samples/scalaproject/api/src/main/scala/org/gradle/sample/api/Person.scala	Sun Oct 26 19:41:33 EST 2008
@@ -0,0 +1,9 @@
+package org.gradle.sample.api
+
+/**
+ * Defines the interface for a person.
+ */
+abstract trait Person
+{
+  def names: List[String]
+}
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaCompileOptions.groovy
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaCompileOptions.groovy	Wed Oct 29 22:22:22 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaCompileOptions.groovy	Wed Oct 29 22:22:22 EST 2008
@@ -0,0 +1,124 @@
+package org.gradle.api.plugins.scala
+
+import org.gradle.api.tasks.compile.AbstractOptions
+
+class ScalaCompileOptions extends AbstractOptions {
+    /**
+     * Whether to use the fsc compile daemon.
+     */
+    boolean useCompileDaemon = false
+
+    // NOTE: Does not work for scalac 2.7.1 due to a bug in the ant task
+    /**
+     * Server (host:port) on which the compile daemon is running.
+     * The host must share disk access with the client process.
+     * If not specified, launches the daemon on the localhost.
+     * This parameter can only be specified if useCompileDaemon is true.
+     */
+    String daemonServer;
+
+    /**
+     * Fail the build on compilation errors.
+     */
+    boolean failOnError = true
+
+    /**
+     * Generate deprecation information.
+     */
+    boolean deprecation = true
+
+    /**
+     * Generate unchecked information.
+     */
+    boolean unchecked = true
+
+    /**
+     * Generate debugging information.
+     * Legal values: none, source, line, vars, notailcalls
+     */
+    String debugLevel
+
+    /**
+     * Run optimisations.
+     */
+    boolean optimize = false
+
+    /**
+     * Encoding of source files.
+     */
+    String encoding = null
+
+    /**
+     * Whether to force the compilation of all files.
+     * Legal values:
+     * - never (only compile modified files)
+     * - changed (compile all files when at least one file is modified)
+     * - always (always recompile all files)
+     */
+    String force = "never"
+
+    /**
+     * Specifies which backend to use.
+     * Legal values: 1.4, 1.5
+     */
+    String targetCompatibility = '1.5'
+
+    /**
+     * Additional parameters passed to the compiler.
+     * Each parameter must start with '-'.
+     */
+    List<String> additionalParameters
+
+    /**
+     * List files to be compiled.
+     */
+    boolean listFiles
+
+    /**
+     * Specifies the amount of logging.
+     * Legal values:  none, verbose, debug
+     */
+    String loggingLevel
+
+    /**
+     * Phases of the compiler to log.
+     * Legal values: namer, typer, pickler, uncurry, tailcalls, transmatch, explicitouter, erasure,
+     *               lambdalift, flatten, constructors, mixin, icode, jvm, terminal.
+     */
+    List<String> loggingPhases
+
+
+    Map fieldName2AntMap() {
+        [
+                failOnError: 'failonerror',
+                loggingLevel: 'logging',
+                loggingPhases: 'logPhase',
+                targetCompatibility: 'target',
+                optimize: 'optimise',
+                daemonServer: 'server',
+                listFiles: 'scalacDebugging',
+                debugLevel: 'debugInfo',
+                additionalParameters: 'addParams'
+        ]
+    }
+
+    Map fieldValue2AntMap() {
+        [
+                deprecation: {toBooleanString(deprecation)},
+                unchecked: {toBooleanString(unchecked)},
+                optimize: {toBooleanString(optimize)},
+                targetCompatibility: {"jvm-${targetCompatibility}"},
+                loggingPhases: {loggingPhases.isEmpty() ? ' ' : loggingPhases.join(',')},
+                additionalParameters: {additionalParameters.isEmpty() ? ' ' : additionalParameters.join(' ')},
+        ]
+    }
+
+    List excludedFieldsFromOptionMap() {
+        ['useCompileDaemon']
+    }
+
+    private String toBooleanString(value) {
+        return value ? 'on' : 'off'
+    }
+
+}
Index: trunk/src/samples/scalaproject/api/src/test/scala/org/gradle/sample/impl/PersonImplTest.scala
===================================================================
--- trunk/src/samples/scalaproject/api/src/test/scala/org/gradle/sample/impl/PersonImplTest.scala	Wed Oct 29 21:54:21 EST 2008
+++ trunk/src/samples/scalaproject/api/src/test/scala/org/gradle/sample/impl/PersonImplTest.scala	Wed Oct 29 21:54:21 EST 2008
@@ -0,0 +1,7 @@
+package org.gradle.sample.impl
+
+class PersonImplTest {
+
+  // FIXME: use a scala test framework to run a test
+
+}
\ No newline at end of file
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDoc.java
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDoc.java	Wed Oct 29 22:05:25 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDoc.java	Wed Oct 29 22:05:25 EST 2008
@@ -0,0 +1,132 @@
+package org.gradle.api.plugins.scala;
+
+import org.gradle.api.*;
+import org.gradle.api.internal.ConventionTask;
+import org.gradle.api.tasks.util.ExistingDirsFilter;
+
+import java.io.File;
+import java.util.ArrayList;
+import java.util.List;
+
+/**
+ * Task to perform scala compilation.
+ */
+public class ScalaDoc extends ConventionTask {
+
+    /**
+     * The directory where to put the generated documentation.
+     */
+    private File destinationDir;
+
+    /**
+     * Directories containing scala source files.
+     */
+    private List<File> scalaSourceDirs;
+
+    /**
+     * Files or directories to add to the compilation classpath.
+     */
+    private List<File> scalaClasspath;
+
+    /**
+     * Include pattern for which scala files should be compiled (e.g. '**&#2F;org/gradle/package1/')).
+     * This pattern is added as an nested include the scalac task.
+     */
+    private List<String> scalaIncludes;
+
+    /**
+     * Exclude pattern for which files should be compiled (e.g. '**&#2F;org/gradle/package2/A*.java').
+     * This pattern is added as an nested exclude the scalac task.
+     */
+    private List<String> scalaExcludes;
+
+    private AntScalaDoc antScalaDoc;
+
+    private ScalaDocOptions scalaDocOptions = new ScalaDocOptions();
+
+    private ExistingDirsFilter existentDirsFilter = new ExistingDirsFilter();
+
+    public ScalaDoc(Project project, String name) {
+        super(project, name);
+        setActions(new ArrayList<TaskAction>());
+        doFirst(new TaskAction() {
+            public void execute(Task task) {
+                generate(task);
+            }
+        });
+    }
+
+    public AntScalaDoc getAntScalaDoc() {
+        if (antScalaDoc == null) {
+            antScalaDoc = new AntScalaDoc(getAnt());
+        }
+        return antScalaDoc;
+    }
+
+    public void setAntScalaDoc(AntScalaDoc antScalaDoc) {
+        this.antScalaDoc = antScalaDoc;
+    }
+
+    public File getDestinationDir() {
+        return (File) conv(destinationDir, "destinationDir");
+    }
+
+    public void setDestinationDir(File destinationDir) {
+        this.destinationDir = destinationDir;
+    }
+
+    public List<File> getScalaSourceDirs() {
+        return (List<File>) conv(scalaSourceDirs, "scalaSourceDirs");
+    }
+
+    public void setScalaSourceDirs(List<File> scalaSourceDirs) {
+        this.scalaSourceDirs = scalaSourceDirs;
+    }
+
+    public List<File> getScalaClasspath() {
+        List<File> files = (List<File>) conv(scalaClasspath, "scalaClasspath");
+
+        // FIXME: remove this - use dependencies
+        List<File> classpath = new ArrayList<File>(files);
+        classpath.add(new File("/usr/java/tools/scala/lib/scala-library.jar"));
+        return classpath;
+    }
+
+    public void setScalaClasspath(List<File> scalaClasspath) {
+        this.scalaClasspath = scalaClasspath;
+    }
+
+    public List<String> getScalaIncludes() {
+        return (List<String>) conv(scalaIncludes, "scalaIncludes");
+    }
+
+    public void setScalaIncludes(List<String> scalaIncludes) {
+        this.scalaIncludes = scalaIncludes;
+    }
+
+    public List<String> getScalaExcludes() {
+        return (List<String>) conv(scalaExcludes, "scalaExcludes");
+    }
+
+    public void setScalaExcludes(List<String> scalaExcludes) {
+        this.scalaExcludes = scalaExcludes;
+    }
+
+    public ScalaDocOptions getScalaDocOptions() {
+        return scalaDocOptions;
+    }
+
+    public void setScalaDocOptions(ScalaDocOptions scalaDocOptions) {
+        this.scalaDocOptions = scalaDocOptions;
+    }
+
+    protected void generate(Task task) {
+
+        List<File> existingSourceDirs = existentDirsFilter.checkDestDirAndFindExistingDirsAndThrowStopActionIfNone(
+                getDestinationDir(), getScalaSourceDirs());
+
+        getAntScalaDoc().execute(existingSourceDirs, getScalaIncludes(), getScalaExcludes(), getDestinationDir(),
+                getScalaClasspath(), getScalaDocOptions());
+    }
+
+}
\ No newline at end of file
Index: trunk/src/samples/scalaproject/build.gradle
===================================================================
--- trunk/src/samples/scalaproject/build.gradle	Sun Oct 26 20:13:53 EST 2008
+++ trunk/src/samples/scalaproject/build.gradle	Sun Oct 26 20:13:53 EST 2008
@@ -0,0 +1,2 @@
+project.group = 'org.gradle'
+project.version = '1.0'
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDocOptions.groovy
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDocOptions.groovy	Wed Oct 29 22:22:22 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDocOptions.groovy	Wed Oct 29 22:22:22 EST 2008
@@ -0,0 +1,77 @@
+package org.gradle.api.plugins.scala
+
+import org.gradle.api.tasks.compile.AbstractOptions
+
+public class ScalaDocOptions extends AbstractOptions {
+
+    /**
+     * Generate deprecation information.
+     */
+    boolean deprecation = true
+
+    /**
+     * Generate unchecked information.
+     */
+    boolean unchecked = true
+
+    /**
+     * Text to appear in the window title.
+     */
+    String windowTitle
+
+    /**
+     * Html text to appear in the main frame title.
+     */
+    String docTitle
+
+    /**
+     * Html text to appear in the header for each page.
+     */
+    String header
+
+    /**
+     * Html text to appear in the footer for each page.
+     */
+    String footer
+
+    /**
+     * Html text to appear in the top text for each page.
+     */
+    String top
+
+    /**
+     * Html text to appear in the bottom text for each page.
+     */
+    String bottom
+
+    /**
+     * Style sheet to override default style.
+     */
+    File styleSheet
+
+    /**
+     * Additional parameters passed to the compiler.
+     * Each parameter must start with '-'.
+     */
+    List<String> additionalParameters
+
+
+    Map fieldName2AntMap() {
+        [
+                additionalParameters: 'addParams'
+        ]
+    }
+
+    Map fieldValue2AntMap() {
+        [
+                deprecation: {toBooleanString(deprecation)},
+                unchecked: {toBooleanString(unchecked)},
+                additionalParameters: {additionalParameters.isEmpty() ? ' ' : additionalParameters.join(' ')},
+        ]
+    }
+
+    private String toBooleanString(value) {
+        return value ? 'on' : 'off'
+    }
+
+}
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaDefine.groovy
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaDefine.groovy	Wed Oct 29 21:07:57 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaDefine.groovy	Wed Oct 29 21:07:57 EST 2008
@@ -0,0 +1,26 @@
+package org.gradle.api.plugins.scala
+
+public class AntScalaDefine {
+
+    private final AntBuilder ant
+    private File scalaLibraryDir;
+
+    def AntScalaDefine(AntBuilder ant, File scalaLibraryDir) {
+        this.ant = ant
+        this.scalaLibraryDir = scalaLibraryDir
+    }
+
+    void execute() {
+
+        File compilerJar = new File(scalaLibraryDir, 'scala-compiler.jar')
+        File libraryJar = new File(scalaLibraryDir, 'scala-library.jar')
+        List<File> taskClasspathFiles = [compilerJar, libraryJar]
+
+        ant.taskdef(resource: 'scala/tools/ant/antlib.xml', loaderRef: 'scala') {
+            taskClasspathFiles.each {file ->
+                classpath(location: file)
+            }
+        }
+    }
+
+}
\ No newline at end of file
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDefine.java
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDefine.java	Wed Oct 29 22:57:36 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaDefine.java	Wed Oct 29 22:57:36 EST 2008
@@ -0,0 +1,55 @@
+package org.gradle.api.plugins.scala;
+
+import org.gradle.api.*;
+import org.gradle.api.internal.ConventionTask;
+import org.gradle.util.GUtil;
+
+import java.io.File;
+
+public class ScalaDefine extends ConventionTask {
+
+    /**
+     * Directory of the installed scala compiler libraries.
+     */
+    private File scalaLibraryDir;
+
+    private AntScalaDefine antScalaDefine;
+
+    public ScalaDefine(Project project, String name) {
+        super(project, name);
+        doFirst(new TaskAction() {
+            public void execute(Task task) {
+                defineAntTasks(task);
+            }
+        });
+    }
+
+    public AntScalaDefine getAntScalaDefine() {
+        if (antScalaDefine == null) {
+            antScalaDefine = new AntScalaDefine(getAnt(), getScalaLibraryDir());
+        }
+        return antScalaDefine;
+    }
+
+    public void setAntScalaDefine(AntScalaDefine antScalaDefine) {
+        this.antScalaDefine = antScalaDefine;
+    }
+
+    public File getScalaLibraryDir() {
+        return (File) conv(scalaLibraryDir, "scalaLibraryDir");
+    }
+
+    public void setScalaLibraryDir(File scalaLibraryDir) {
+        this.scalaLibraryDir = scalaLibraryDir;
+    }
+
+    protected void defineAntTasks(Task task) {
+
+        if (!GUtil.isTrue(getScalaLibraryDir())) {
+            throw new InvalidUserDataException("The scalaLibraryDir must be set!");
+        }
+
+        getAntScalaDefine().execute();
+    }
+
+}
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaPluginConvention.groovy
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaPluginConvention.groovy	Wed Oct 29 22:22:22 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/ScalaPluginConvention.groovy	Wed Oct 29 22:22:22 EST 2008
@@ -0,0 +1,46 @@
+package org.gradle.api.plugins.scala
+
+import org.gradle.api.Project
+import org.gradle.api.internal.plugins.PluginUtil
+import org.gradle.api.plugins.JavaPluginConvention
+
+
+class ScalaPluginConvention {
+
+    Project project
+    File scalaLibraryDir
+    List scalaSrcDirNames = []
+    List scalaTestSrcDirNames = []
+    List floatingScalaSrcDirs = []
+    List floatingScalaTestSrcDirs = []
+    String scalaDocDirName
+
+    ScalaPluginConvention(Project project, Map customValues) {
+        this.project = project
+        scalaSrcDirNames << 'main/scala'
+        scalaTestSrcDirNames << 'test/scala'
+        scalaDocDirName = 'scaladoc'
+        PluginUtil.applyCustomValues(project.convention, this, customValues)
+    }
+
+    File getScalaLibraryDir() {
+        return scalaLibraryDir
+    }
+
+    List getScalaSrcDirs() {
+        scalaSrcDirNames.collect {new File(javaConvention.srcRoot, it)} + floatingScalaSrcDirs
+    }
+
+    List getScalaTestSrcDirs() {
+        scalaTestSrcDirNames.collect {new File(javaConvention.srcRoot, it)} + floatingScalaTestSrcDirs
+    }
+
+    File getScalaDocDir() {
+        return new File(javaConvention.docsDir, scalaDocDirName)
+    }
+
+    private JavaPluginConvention getJavaConvention() {
+        project.convention.plugins.java
+    }
+
+}
Index: trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaDoc.groovy
===================================================================
--- trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaDoc.groovy	Wed Oct 29 22:22:22 EST 2008
+++ trunk/src/main/groovy/org/gradle/api/plugins/scala/AntScalaDoc.groovy	Wed Oct 29 22:22:22 EST 2008
@@ -0,0 +1,74 @@
+package org.gradle.api.plugins.scala
+
+import org.slf4j.Logger
+import org.slf4j.LoggerFactory
+
+
+class AntScalaDoc {
+    private static Logger logger = LoggerFactory.getLogger(AntScalaDoc)
+
+    private final AntBuilder ant
+    private final List<File> bootclasspathFiles
+    private final List<File> extensionDirs
+
+    def AntScalaDoc(AntBuilder ant) {
+        this.ant = ant
+        this.bootclasspathFiles = []
+        this.extensionDirs = []
+    }
+
+    def AntScalaDoc(AntBuilder ant, List<File> bootclasspathFiles, List<File> extensionDirs) {
+        this.ant = ant
+        this.bootclasspathFiles = bootclasspathFiles
+        this.extensionDirs = extensionDirs
+    }
+
+    void execute(List<File> sourceDirs, List<String> includes, List<String> excludes, File targetDir,
+                 List<File> classpathFiles, ScalaDocOptions docOptions) {
+
+        ant.mkdir(dir: targetDir.absolutePath)
+
+        Map options = ['destDir': targetDir] + docOptions.optionMap()
+        if (logger.isInfoEnabled()) {
+            StringBuilder builder = new StringBuilder()
+            builder.append("Generating scaladoc from source ${sourceDirs}\n")
+            builder.append("    options = ${options}\n")
+            builder.append("    classpath = ${classpathFiles}\n")
+            if (includes) {
+                builder.append("    includes = ${includes}\n")
+            }
+            if (excludes) {
+                builder.append("    excludes = ${excludes}\n")
+            }
+            if (bootclasspathFiles) {
+                builder.append("    bootclasspath = ${bootclasspathFiles}\n")
+            }
+            if (extensionDirs) {
+                builder.append("    extDirs = ${extensionDirs}\n")
+            }
+            logger.info(builder.toString())
+        }
+
+        ant.scaladoc(options) {
+            sourceDirs.each {dir ->
+                src(location: dir)
+            }
+            bootclasspathFiles.each {file ->
+                bootclasspath(location: file)
+            }
+            extensionDirs.each {dir ->
+                extdirs(location: dir)
+            }
+            classpathFiles.each {file ->
+                classpath(location: file)
+            }
+            includes.each {pattern ->
+                include(name: pattern)
+            }
+            excludes.each {pattern ->
+                exclude(name: pattern)
+            }
+        }
+    }
+
+}
